/*
 *  Copyright 2014 (c) Martin Klapetek <mklapetek@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef ABSTRACTOSD_H
#define ABSTRACTOSD_H

#include "notificationmanager_export.h"

#include <QObject>
#include <QString>

#include <KSharedConfig>

class QTimer;

namespace NotificationManager
{

class NOTIFICATIONMANAGER_EXPORT AbstractOsd : public QObject
{
    Q_OBJECT

public:
    enum class Mode {
        Standart = 0,
        Rounded
    };

    AbstractOsd(const KSharedConfig::Ptr &config,
                QObject *parent = nullptr,
                Mode mode = Mode::Standart);
    ~AbstractOsd() override;

    static bool registerOSDService();

public Q_SLOTS:
    void brightnessChanged(int percent);
    void keyboardBrightnessChanged(int percent);
    void volumeChanged(int percent);
    void microphoneVolumeChanged(int percent);
    void mediaPlayerVolumeChanged(int percent, const QString &playerName, const QString &playerIconName);
    void kbdLayoutChanged(const QString &layoutName);
    void virtualDesktopChanged(const QString &currentVirtualDesktopName);
    void touchpadEnabledChanged(bool touchpadEnabled);
    void wifiEnabledChanged(bool wifiEnabled);
    void bluetoothEnabledChanged(bool bluetoothEnabled);
    void wwanEnabledChanged(bool wwanEnabled);
    void virtualKeyboardEnabledChanged(bool virtualKeyboardEnabled);
    void showText(const QString &icon, const QString &text);
    void showProgressBar(const QString &icon, const int percent);

Q_SIGNALS:
    void osdProgress(const QString &icon, const int percent, const QString &additionalText);
    void osdText(const QString &icon, const QString &text);

protected:
    virtual bool init();

    virtual QObject *osdRootObject() const = 0;

    void setTimeout(int timeout);
    int timeout() const;

    virtual void updateTimeoutFormOsdRootObject();

    KSharedConfig::Ptr config() const;

    Mode mode() const;

private Q_SLOTS:
    void hideOsd();

private:
    void showProgress(const QString &icon, const int percent, const QString &additionalText = QString());
    void showOsd();

    QTimer *m_osdTimer = nullptr;
    int m_timeout = 1800;

    KSharedConfig::Ptr m_config;

    Mode m_mode;
};

}

#endif // ABSTRACTOSD_H
